# -*- coding: utf-8 -*-
from .putshebang import ShebangedFile, UnshebangedFile, shebang, _ShebangNotFoundException
"""Top-level package for putshebang."""

__all__ = ["ShebangedFile", "UnshebangedFile", "shebang", "_ShebangNotFoundException"]
__author__ = """Fady Adel"""
__email__ = '2masadel@gmail.com'
__version__ = '0.1.3'

